import csv


def get_junctions(vecFile):
    subcatsDs = {}
    masterSubcats = []
    lengths = {}
    slopes = {}
    prints = []
    with open(vecFile, 'r') as csvfile:
        reader = csv.reader(csvfile)
        subcats = []
        lastSubcat = None
        for rows in reader:
            if not len(rows) > 0:
                continue
            if rows[0][:5].lower() == "print":
                prints.extend([lastSubcat])
                continue
            if rows[0][:5].lower() in ["rain ", "add r", "route"]:
                subcat = rows[0].split('#')[1].split()[0]
                lastSubcat = subcat
                if subcat not in masterSubcats:
                    masterSubcats.extend([subcat])
                if rows[0][:5].lower() in ["rain ", "add r"]:
                    lengths[subcat] = rows[0].split()[rows[0].split().index('L')+2]
                    slopes[subcat] = rows[0].split()[rows[0].split().index('Sc')+2]
                if rows[0][:5].lower() == "route":
                    newSubcats = []
                    for subby in subcats:
                        if type(subby) is str:
                            subcatsDs[subby] = subcat
                        else:
                            newSubcats.extend(subby)
                    subcats = newSubcats
                else:
                    if subcat not in subcats:
                        subcats.extend([subcat])
    return masterSubcats, subcatsDs, prints, lengths, slopes
            
def get_centroids(subcatsFile):
    subcatLayer = QgsVectorLayer(subcatsFile, "Subbies")
    sourceCrs = subcatLayer.crs()
    targetCrs = QgsCoordinateReferenceSystem('EPSG:4326')
    coord_transform = QgsCoordinateTransform(sourceCrs, targetCrs, QgsCoordinateTransformContext())
    centroids = {}
    areas = {}
    for feature in subcatLayer.getFeatures():
        geom = feature.geometry()
        centroid = coord_transform.transform(geom.centroid().asPoint())
        x = centroid.x()
        y = centroid.y()
        centroids[str(feature.attributes()[0])] = [x,y]
        areas[str(feature.attributes()[0])] = feature.attributes()[1]
    return centroids, areas
        
        
def convert_to_RORB(outputPath, subcatsFile, vecFile):
    
    centroids, areas = get_centroids(subcatsFile)
    subcats, subbyDS, prints, lengths, slopes = get_junctions(vecFile)
    numReaches = 0
    with open(outputPath, 'w') as f:
        f.write("HydroRepo\n")
        f.write("C RORB_GE 6.32\n")
        f.write("C WARNING - DO NOT EDIT THIS FILE OUTSIDE RORB TO ENSURE BOTH GRAPHICAL AND CATCHMENT DATA ARE COMPATIBLE WITH EACH OTHER\n")
        f.write("C THIS FILE CANNOT BE OPENED IN EARLIER VERSIONS OF RORB GE - CURRENT VERSION IS v6.32\n")
        f.write("C \n")
        f.write("C HydroRepo\n")
        f.write("C\n")
        f.write("C #FILE COMMENTS\n")
        f.write("C   0\n")
        f.write("C \n")
        f.write("C #SUB-AREA AREA COMMENTS\n")
        f.write("C   0\n")
        f.write("C \n")
        f.write("C #IMPERVIOUS FRACTION COMMENTS\n")
        f.write("C   0\n")
        f.write("C \n")
        f.write("C #BACKGROUND IMAGE\n")
        f.write("C  T  F\n")
        f.write("C\n")
        f.write("C #NODES\n")
        
        subcats.sort(key=int)
        f.write(f"C{str(len(subcats) + len(subbyDS)).rjust(7)}\n")
        for subcat in subcats:
            line = "C"
            line += subcat.rjust(7)
            line += str(round(centroids[subcat][0],3)).rjust(15)
            line += str(round(centroids[subcat][1],3)).rjust(15)
            line += "1.000".rjust(15)
            line += "1 0".rjust(4)
            line += str(len(subcats) + int(subcat)).rjust(6)
            line += ' ' + str(subcat)
            line += areas[subcat].rjust(33 - len(' ' + str(subcat)))
            line += "0.000".rjust(15)
            if (subcat in prints):
                line += "0".rjust(3)
                line += "  0  0"
                line += '\n'
                line += f"C  Inflow {subcat}"
            else:
                line += "0".rjust(3)
                line += "  0  0"
                line += '\n'
                line += f"C"
            line += '\n'
            f.write(line)
        
        
        for subcat in subcats:
            line = "C"
            line += str(len(subcats) + int(subcat)).rjust(7)
            line += str(round(centroids[subcat][0],3)).rjust(15)
            line += str(round(centroids[subcat][1],3)).rjust(15)
            line += "1.000".rjust(15)
            if subcat in subbyDS:
                line += "0 0".rjust(4)
                line += subbyDS[subcat].rjust(6)
            else:
                line += "0 1".rjust(4)
                line += "0".rjust(6)
            line += ' ' + f"J-{subcat}"
            line += areas[subcat].rjust(33 - len(' ' + f"J-{subcat}"))
            line += "0.000".rjust(15)
            line += "70".rjust(3)
            line += "  0  0"
            line += '\n'
            line += f"C  Inflow J-{subcat}"
            line += '\n'
            
            f.write(line)
            
        f.write("C\n")
        f.write("C #REACHES\n")
        f.write("C" + str(len(subcats) + len(subbyDS)).rjust(7) + "\n")
        
        for subcat in subcats:
            line = "C"
            line += subcat.rjust(7)
            line += ' R' + subcat
            line += subcat.rjust(26 - len('R' + subcat))
            line += str(int(subcat) + len(subcats)).rjust(6)
            line += "              0 1 0"
            line += lengths[subcat].rjust(15)
            line += slopes[subcat].rjust(15)
            line += "     1  0"
            line += '\n'
            line += 'C'
            line += str(round(centroids[subcat][0],3)).rjust(16) + '\n'
            line += 'C'
            line += str(round(centroids[subcat][1],3)).rjust(16) + '\n'
            
            f.write(line)
            
        for subcat in subcats:
            if not subcat in subbyDS:
                continue
            line = "C"
            line += str(len(subcats) + int(subcat)).rjust(7)
            line += ' R' + str(len(subcats) + int(subcat))
            line += str(int(subcat) + len(subcats)).rjust(26 - len('R' + str(len(subcats) + int(subcat))))
            line += subbyDS[subcat].rjust(6)
            line += "              0 1 0"
            line += lengths[subbyDS[subcat]].rjust(15)
            line += slopes[subbyDS[subcat]].rjust(15)
            line += "     1  0"
            line += '\n'
            line += 'C'
            line += str(round(centroids[subbyDS[subcat]][0],3)).rjust(16) + '\n'
            line += 'C'
            line += str(round(centroids[subbyDS[subcat]][1],3)).rjust(16) + '\n'
            
            f.write(line)
            
        f.write("C\n")
        f.write("C #STORAGES\n")
        f.write("C      0\n")
        f.write("C\n")
        f.write("C #INFLOW/OUTFLOW\n")
        f.write("C      0\n")
        f.write("C\n")
        f.write("C END RORB_GE\n")
            
            
        
convert_to_RORB(r"F:\scripts\Mydro\Examples\EPR\Model\RORB\rorb.catg",
                r"R:\Jobs\24020147_Logan_Flood_Study_2024_Flagstone_Creek\Analysis\CSIM\003_export\Flagstone_03-SubCatchments.shp",
                r"R:\Jobs\24020147_Logan_Flood_Study_2024_Flagstone_Creek\Analysis\CSIM\003_export\URBS\FLAG_03.vec")